export interface Skill {
  name: string
  link?: string
  tags: string[]
  detail?: string
}

export interface SkillsState {
  currentTag: string | null
  descriptions: boolean
  skills: Skill[]
  tags: string[]
  visibleSkills: Skill[]

  init(): void
  setCurrentTag(e: MouseEvent): void
  tagClass(value: string): string
  toggleDescriptions(e: MouseEvent): void
}

export default function Skills(): SkillsState {
  function readCurrentTag() {
    const usp = new URLSearchParams(window.location.search)
    return usp.get('skill')
  }

  function readCurrentDescriptions() {
    const usp = new URLSearchParams(window.location.search)
    return usp.get('descriptions') === '1'
  }

  function updateVisibleSkills(this: SkillsState) {
    const tag = this.currentTag
    if (tag) this.visibleSkills = this.skills.filter(skill => skill.tags.includes(tag))
    else this.visibleSkills = this.skills
  }

  return {
    currentTag: null,
    descriptions: false,
    skills: [],
    visibleSkills: [],

    get tags() {
      return this.skills
        .reduce((tags, skill) => {
          for (const tag of skill.tags) {
            if (tags.indexOf(tag) === -1) tags.push(tag)
          }
          return tags
        }, <string[]>[])
        .sort()
    },

    async init() {
      this.currentTag = readCurrentTag()
      this.descriptions = readCurrentDescriptions()

      try {
        const res = await fetch(`//${document.location.host}/data/skills.json`)
        if (res.status !== 200) throw new Error('not OK')

        const data = await res.json() as Skill[]
        this.skills = (data).sort((a, b) => a.name.localeCompare(b.name))
        updateVisibleSkills.apply(this)
      } catch (err) {
        const e = err as Error
        console.error(e.name, e.message)
      }
    },

    setCurrentTag(e) {
      const tag = (e.target as HTMLAnchorElement)?.dataset?.tag
      if (!tag) return

      e.preventDefault()

      const usp = new URLSearchParams(window.location.search)
      if (tag === this.currentTag) {
        this.currentTag = null
        usp.delete('skill')
      } else {
        this.currentTag = tag
        usp.set('skill', tag)
      }

      const search = usp.toString()
      if (search) window.history.pushState({ tag }, '', `${window.location.pathname}?${search}`)
      else window.history.pushState({ tag }, '', window.location.pathname)

      updateVisibleSkills.apply(this)
      return tag
    },

    tagClass(value: string) {
      if (value === this.currentTag) return 'tag active'
      return 'tag'
    },

    toggleDescriptions(e) {
      e.preventDefault()

      const usp = new URLSearchParams(window.location.search)
      if (this.descriptions) usp.delete('descriptions')
      else usp.set('descriptions', '1')

      const search = usp.toString()
      if (search) window.history.pushState(null, '', `${window.location.pathname}?${search}`)
      else window.history.pushState(null, '', window.location.pathname)

      this.descriptions = !this.descriptions
    },
  }
}
